@@ -25,7 +25,7 @@ BEGIN
   our %EXPORT_TAGS = ( go => [@EXPORT_OK] );
 }
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 require App::MechaCPAN::Perl;
 require App::MechaCPAN::Install;
@@ -42,17 +42,13 @@ INIT
 
 $loaded_at_compile //= 0;
 
-my @args = (
+our @args = (
   'diag-run!',
-  @App::MechaCPAN::Perl::args,
-  @App::MechaCPAN::Install::args,
-  @App::MechaCPAN::Deploy::args,
   'verbose|v!',
   'quiet|q!',
   'no-log!',
   'directory|d=s',
 );
-@args = keys %{ { map { $_ => 1 } @args } };
 
 # Timeout when there's no output in seconds
 our $TIMEOUT = $ENV{MECHACPAN_TIMEOUT} // 30;
@@ -83,6 +79,14 @@ sub main
     }
   }
 
+  my @args = (
+    @App::MechaCPAN::args,
+    @App::MechaCPAN::Perl::args,
+    @App::MechaCPAN::Install::args,
+    @App::MechaCPAN::Deploy::args,
+  );
+  @args = keys %{ { map { $_ => 1 } @args } };
+
   my $options = {};
   my $getopt_ret
     = Getopt::Long::GetOptionsFromArray( \@argv, $options, @args );
@@ -95,8 +99,6 @@ sub main
     if ( ref $arg eq 'HASH' )
     {
       $options = { %$arg, %$options };
-
-      #say Data::Dumper::Dumper($arg, $options);
       return 0;
     }
     return 1;
@@ -115,6 +117,9 @@ sub main
   }
 
   local $LOGFH;
+  local $VERBOSE = $options->{verbose} // $VERBOSE;
+  local $QUIET   = $options->{quiet}   // $QUIET;
+
   my $dest_dir = &dest_dir;
   my $cmd      = ucfirst lc shift @argv;
   my $pkg      = join( '::', __PACKAGE__, $cmd );
@@ -233,6 +238,13 @@ sub _show_line
   }
   $idx++;
 
+  # Don't bother with fancy line movements if we are verbose
+  if ($VERBOSE)
+  {
+    print STDERR "$color$line$RESET\n";
+    return;
+  }
+
   # We use some ANSI escape codes, so they are:
   # \e[.F  - Move up from current line, which is always the end of the list
   # \e[K   - Clear the line
@@ -268,7 +280,7 @@ sub status
   state @last_key;
 
   # Undo the last line that is bold
-  if (@last_key)
+  if ( @last_key && !$VERBOSE )
   {
     _show_line(@last_key);
   }
@@ -402,7 +414,7 @@ sub run
   }
 
   # If the output is asked for (non-void context), don't show it anywhere
-  if ( $wantoutput )
+  if ($wantoutput)
   {
     open $dest_out_fh, ">", \$out;
     open $dest_err_fh, ">", \$err;
@@ -413,10 +425,10 @@ sub run
   my ( $error,  $error_chld )  = _genio;
 
   warn( join( "\t", $cmd, @args ) . "\n" )
-      if $VERBOSE;
+    if $VERBOSE;
 
   print $dest_err_fh ( 'Running: ', join( "\t", $cmd, @args ) . "\n" )
-      if defined $dest_err_fh;
+    if defined $dest_err_fh;
 
   my $pid = open3(
     undef,
@@ -455,17 +467,17 @@ sub run
         if ( $fh eq $output )
         {
           print $dest_out_fh $line
-              if defined $dest_out_fh;
+            if defined $dest_out_fh;
           $out .= $line
-              unless $wantoutput;
+            unless $wantoutput;
         }
 
         if ( $fh eq $error )
         {
           print $dest_err_fh $line
-              if defined $dest_err_fh;
+            if defined $dest_err_fh;
           $err .= $line
-              unless $wantoutput;
+            unless $wantoutput;
         }
 
       }
@@ -475,7 +487,7 @@ sub run
   my $error = $@;
   alarm 0;
 
-  if ($error eq $alrm_code)
+  if ( $error eq $alrm_code )
   {
     info "Idle timeout (${TIMEOUT}s) exceeded, killing";
     kill "KILL", $pid;
@@ -490,22 +502,22 @@ sub run
     my $core = $? & 128 ? 'Core Dumped' : '';
 
     croak ""
-        . Term::ANSIColor::color('RED')
-        . qq/\nCould not execute '/
-        . join( ' ', $cmd, @args ) . qq/'/
-        . qq/\nPID: $pid/
-        . qq/\t$code/
-        . qq/\t$sig/
-        . qq/\t$core/
-        . Term::ANSIColor::color('GREEN')
-        . qq/\n$out/
-        . Term::ANSIColor::color('YELLOW')
-        . qq/\n$err/
-        . Term::ANSIColor::color('RESET') . "\n";
+      . Term::ANSIColor::color('RED')
+      . qq/\nCould not execute '/
+      . join( ' ', $cmd, @args ) . qq/'/
+      . qq/\nPID: $pid/
+      . qq/\t$code/
+      . qq/\t$sig/
+      . qq/\t$core/
+      . Term::ANSIColor::color('GREEN')
+      . qq/\n$out/
+      . Term::ANSIColor::color('YELLOW')
+      . qq/\n$err/
+      . Term::ANSIColor::color('RESET') . "\n";
   }
 
   return
-      if !defined wantarray;
+    if !defined wantarray;
 
   if (wantarray)
   {
